package com.example.ellegadodepintia

import android.graphics.Color
import android.os.Bundle
import android.view.View
import android.widget.Button
import android.widget.ImageView
import android.widget.TextView
import androidx.appcompat.app.AppCompatActivity
import com.example.ellegadodepintia.exploradoresDePintia.ModalDetallesCompra
import com.example.ellegadodepintia.exploradoresDePintia.model.GameState
import com.example.ellegadodepintia.exploradoresDePintia.model.LayoutUtils
import com.example.ellegadodepintia.exploradoresDePintia.model.objeto.Objeto
import com.example.ellegadodepintia.repositorios.RepositorioJugador
import com.example.ellegadodepintia.repositorios.RepositorioObjetos

class ArmeriaActivity : AppCompatActivity() {
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_armeria)

        GameState.jugador.addObserver { LayoutUtils.actualizarMonedasTienda(this)
            cargarObjetos()}

        val nivelCiudad = GameState.jugador.nivelCiudad
        val nObjetos = nivelCiudad * 2

        val texto = findViewById<TextView>(R.id.textoInformacion)

        val botonRefrescar = findViewById<Button>(R.id.botonRefrescar)
        botonRefrescar.setOnClickListener {
            if(GameState.jugador.monedas >= 5){
                GameState.jugador.actualizarObjetosArmeria(nObjetos)
                RepositorioJugador.setMonedas(GameState.jugador.monedas - 5)
                GameState.jugador.actualizarMonedas(GameState.jugador.monedas - 5)
                "Se ha actualizado la tienda".also { texto.text = it }
            } else {
                "No tienes suficientes monedas para refrescar".also { texto.text = it }
            }

        }

        val textoMonedas = findViewById<TextView>(R.id.textoMonedas)
        GameState.jugador.monedas.toString().also { textoMonedas.text = it }
        cargarObjetos()
    }

    private fun cargarObjetos(){

        val nivelCiudad = GameState.jugador.nivelCiudad
        val nObjetos = nivelCiudad * 2
        val objetos = RepositorioObjetos.convertirNombresAObjetos(GameState.jugador.objetosArmeria)
        val preciosGenerados = ArrayList<Int>()

        val slots = listOf<ImageView>(
            findViewById(R.id.slot01),
            findViewById(R.id.slot02),
            findViewById(R.id.slot03),
            findViewById(R.id.slot04),
            findViewById(R.id.slot05),
            findViewById(R.id.slot06)
        )

        val precios = listOf<TextView>(
            findViewById(R.id.precio01),
            findViewById(R.id.precio02),
            findViewById(R.id.precio03),
            findViewById(R.id.precio04),
            findViewById(R.id.precio05),
            findViewById(R.id.precio06)
        )

        val monedas = listOf<ImageView>(
            findViewById(R.id.moneda01),
            findViewById(R.id.moneda02),
            findViewById(R.id.moneda03),
            findViewById(R.id.moneda04),
            findViewById(R.id.moneda05),
            findViewById(R.id.moneda06)
        )

        precios.forEachIndexed { index, precio ->
            if (index < nObjetos) {
                val resultado = (30..75).random() / nivelCiudad
                preciosGenerados.add(resultado)
                resultado.toString().also { precio.text = it }
            } else {
                val nivelTaberna = index / 2 + 1
                "Taberna lvl $nivelTaberna".also { precio.text = it }
                precio.textSize = 11f
                precio.setTextColor(Color.RED)

            }
        }

        slots.forEachIndexed { index, slot ->
            if (index < nObjetos) {
                slot.setImageResource(objetos[index]!!.imagen)
                slot.setOnClickListener {
                    val modalDetallesFragment = ModalDetallesCompra().apply {
                        objeto = objetos[index]
                        coste = preciosGenerados[index]
                    }

                    modalDetallesFragment.onDismissListener = { accionRealizada ->
                        if (accionRealizada) {
                            ejecutarCompra(objetos[index], preciosGenerados[index])
                        }
                    }

                    modalDetallesFragment.show(supportFragmentManager, "modalDetalles")
                }
            } else {
                slot.setImageResource(R.drawable.asi_candado_tienda)
                slot.scaleType = ImageView.ScaleType.CENTER
            }
        }

        monedas.forEachIndexed { index, moneda ->
            if (index >= nObjetos) {
                moneda.visibility = View.GONE
            }
        }
    }

    private fun ejecutarCompra(objeto: Objeto?, coste: Int){
        val texto = findViewById<TextView>(R.id.textoInformacion)
        if (GameState.jugador.monedas < coste){
            "No hay dinero".also { texto.text = it }
        }else{
            GameState.jugador.actualizarMonedas(GameState.jugador.monedas - coste)
            RepositorioJugador.setMonedas(GameState.jugador.monedas)
            if (objeto != null) {
                GameState.jugador.conseguirObjeto(objeto)
            }
            "Objeto comprado y añadido al inventario".also { texto.text = it }
        }
    }
}
