package com.example.ellegadodepintia.exploradoresDePintia

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.graphics.Color
import android.graphics.drawable.AnimationDrawable
import android.os.Bundle
import android.widget.ImageButton
import android.widget.ImageView
import android.widget.Toast
import androidx.activity.result.ActivityResultLauncher
import androidx.activity.result.contract.ActivityResultContracts
import androidx.appcompat.app.AppCompatActivity
import com.example.ellegadodepintia.R
import com.example.ellegadodepintia.exploradoresDePintia.model.Atributo
import com.example.ellegadodepintia.exploradoresDePintia.model.GameState

import com.example.ellegadodepintia.exploradoresDePintia.model.GeneradorDeEventos
import com.example.ellegadodepintia.exploradoresDePintia.model.LayoutUtils
import com.example.ellegadodepintia.exploradoresDePintia.model.GestorDeEventos
import com.example.ellegadodepintia.repositorios.RepositorioObjetos
import com.example.ellegadodepintia.soundManager.SoundManager
import java.lang.ref.WeakReference

class ExploradoresDePintia : AppCompatActivity() {
    private lateinit var soundManager: SoundManager
    private var dificultad = 0
    private lateinit var startForResult: ActivityResultLauncher<Intent>


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_exploradores_de_pintia)

        soundManager = SoundManager(this)
        soundManager.playSoundLoop(R.raw.sound_exploradores_idle,100)
        startForResult =
            registerForActivityResult(ActivityResultContracts.StartActivityForResult()) { result ->
                if (result.resultCode == Activity.RESULT_OK) {
                    val mensajeResultado = result.data?.getStringExtra("resultado")
                    println("Volvemos del mini-juego: $mensajeResultado")
                }
            }

        GameState.fragmentManager = supportFragmentManager

        GeneradorDeEventos.generarEventos()

        GameState.setContexto(WeakReference<Context>(this))

        val inventoryButton: ImageButton = findViewById(R.id.inventoryButton)
        val eventButton: ImageButton = findViewById(R.id.eventButton)

        eventButton.setBackgroundResource(R.drawable.style_texto_border)
        inventoryButton.setBackgroundColor(Color.TRANSPARENT)

        inventoryButton.setOnClickListener {
            val inventarioFragment = InventoryMain()
            inventarioFragment.show(supportFragmentManager, "inventario")
        }

        val imagenProtagonista = findViewById<ImageView>(R.id.imagenProtagonista)
        val idleAnimation = imagenProtagonista.drawable as? AnimationDrawable
        idleAnimation?.start()

        iniciarJuego()
    }

    private fun iniciarJuego() {
        GameState.jugador.addObserver {
            LayoutUtils.actualizarEstadisticasMain()
            LayoutUtils.actualizarMonedas()
        }
        LayoutUtils.actualizarEstadisticasMain()
        LayoutUtils.actualizarMonedas()
        GeneradorDeEventos.generarEvento(dificultad)
        GestorDeEventos.gestionar()
    }

    @Deprecated("Deprecated in Java")
    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == 100 && resultCode == RESULT_OK) {
            val resultado = data?.getStringExtra("resultado")
            Toast.makeText(this, resultado, Toast.LENGTH_SHORT).show()

        } else if (requestCode == 102 && resultCode == RESULT_OK) {
            val puntuacion = data?.getStringExtra("PUNTUACION")!!.toInt()
            GameState.jugador.actualizarAtributo(Atributo.Energia, puntuacion/3)
            GameState.jugador.actualizarAtributo(Atributo.Vida, puntuacion/3)

        } else if (requestCode == 103 && resultCode == RESULT_OK) {
            val tiempo = data?.getStringExtra("TIEMPO")!!.toInt()
            if (tiempo >= 1000){
                GameState.jugador.actualizarMonedas(GameState.jugador.monedas.plus(tiempo/1000))
            }else if (GameState.jugador.inventario.size == 0){
                if (GameState.jugador.atributos[Atributo.Energia] == 1){
                    GameState.jugador.actualizarAtributo(Atributo.Energia, -1)
                }else{
                    GameState.jugador.actualizarAtributo(Atributo.Energia, -GameState.jugador.atributos[Atributo.Energia]!!/2)
                }
            }else{
                GameState.jugador.destruirObjetoRandom()
            }
        }else if (requestCode == 104 && resultCode == RESULT_OK) {
            val puntosHuesos = data?.getStringExtra("PUNTOS_HUESOS")!!.toInt()
            val objeto= RepositorioObjetos.obtenerObjetoPorCoste(puntosHuesos)
            if (objeto != null){
                GameState.jugador.conseguirObjeto(objeto)
            }
        }

    }

    override fun onDestroy() {
        super.onDestroy()
        soundManager.stopSound()
    }


}
