package com.example.ellegadodepintia.exploradoresDePintia.model

import android.graphics.drawable.AnimationDrawable
import android.widget.ImageView
import android.widget.TextView
import com.example.ellegadodepintia.R
import com.example.ellegadodepintia.exploradoresDePintia.model.GameState.context
import com.example.ellegadodepintia.exploradoresDePintia.model.eventoFactory.Evento
import com.example.ellegadodepintia.exploradoresDePintia.model.eventoFactory.EventoCombate
import com.example.ellegadodepintia.exploradoresDePintia.model.eventoFactory.EventoFactorySelector
import kotlin.random.Random


object GeneradorDeEventos {
    private const val CANTIDAD = 10 // Numero de eventos generados
    private var eventos = ArrayList<Evento>()

    private val pesosBase = mutableMapOf(
        TipoEvento.Reliquia to 0.25,
        TipoEvento.Tienda to 0.1,
        TipoEvento.Riesgo to 0.2,
        TipoEvento.Trampa to 0.1,
        TipoEvento.Combate to 0.25,
        TipoEvento.Descanso to 0.0,
        TipoEvento.Ladron to 0.1
    )

    private var historialTipos = mutableListOf<TipoEvento>()


    fun generarEventos() {
        val eventosGenerados = ArrayList<Evento>()
        historialTipos = mutableListOf()


        while (eventosGenerados.size < CANTIDAD) {
            val tipoSeleccionado = seleccionarTipoEvento()
            println(historialTipos)
            if (historialTipos.isNotEmpty() && tipoSeleccionado == historialTipos.last()) {
                continue
            }

            val nuevoEvento = EventoFactorySelector.obtenerFactory(tipoSeleccionado).generarEvento()

            if (!eventosGenerados.contains(nuevoEvento)) {
                eventosGenerados.add(nuevoEvento)
                historialTipos.add(tipoSeleccionado)
            }
        }

        eventos = eventosGenerados
    }

    private fun seleccionarTipoEvento(): TipoEvento {
        val pesosActuales = pesosBase.toMutableMap()

        // 20% Descanso tras dos riegsos en los últimos 3 eventos
        val recientes = historialTipos.takeLast(3)
        if (recientes.count { it == TipoEvento.Riesgo } >= 2) {
            pesosActuales[TipoEvento.Tienda] = pesosActuales[TipoEvento.Tienda]!! + 0.2
        // 25% Ladron si 2 reliquias
        }else if(recientes.count{it == TipoEvento.Reliquia} >= 2){
            pesosActuales[TipoEvento.Ladron] = 0.25
        }

        // 35% Prob de descanso tras combate 20% Tienda 10% Combate
        val ultimoTipo = historialTipos.lastOrNull()
        if (ultimoTipo == TipoEvento.Combate) {
            pesosActuales[TipoEvento.Descanso] = 0.35
            pesosActuales[TipoEvento.Tienda] = 0.2
            pesosActuales[TipoEvento.Combate] = 0.1
        }

        val totalPesos = pesosActuales.values.sum()
        val probabilidadesNormalizadas = pesosActuales.mapValues { it.value / totalPesos }

        val acumulado = probabilidadesNormalizadas.entries.fold(mutableListOf<Pair<Double, TipoEvento>>()) { acc, entry ->
            val sumaAcumulada = acc.lastOrNull()?.first ?: 0.0
            acc.add(Pair(sumaAcumulada + entry.value, entry.key))
            acc
        }

        val random = Random.nextDouble()
        val tipoSeleccionado = acumulado.first { random <= it.first }.second

        return tipoSeleccionado
    }

    fun comprobarEventoDisponible() : Boolean{
        return eventos.size  >= 1
    }

    fun generarEvento(dificultad: Int) {
        "Resultado del evento".also { context!!.findViewById<TextView>(R.id.textoResultado).text = it }

        val eventoSeleccionado = filtrarEventosPorDificultad(eventos, dificultad)
        eventos.remove(eventoSeleccionado)

        val textEvento = context!!.findViewById<TextView>(R.id.textoEvento)
        textEvento.text = eventoSeleccionado.descripcion
        val imagenEvento = context!!.findViewById<ImageView>(R.id.imagenEvento)

        imagenEvento.setImageResource(eventoSeleccionado.imagen)

        val drawable = imagenEvento.drawable

        if (drawable is AnimationDrawable && eventoSeleccionado is EventoCombate) {
            drawable.start()
        }

        if(eventoSeleccionado is EventoCombate){
            GestorCombate.evento = eventoSeleccionado
        }
        GameState.eventoActual = eventoSeleccionado
        GameState.eventoActual.addObserver {
            LayoutUtils.actualizarEstadisticasEvento() }

        LayoutUtils.actualizarEstadisticasEvento()

    }

    private fun filtrarEventosPorDificultad(eventos: ArrayList<Evento>, dificultad: Int): Evento {
        val eventosFiltrados = eventos.filter { it.dificultad == dificultad }
        return eventosFiltrados[eventosFiltrados.indices.first]
    }


}
