package com.example.ellegadodepintia.exploradoresDePintia.model.eventoFactory

import com.example.ellegadodepintia.R
import com.example.ellegadodepintia.exploradoresDePintia.model.Atributo
import com.example.ellegadodepintia.exploradoresDePintia.model.Opcion
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.EfectoIgnorar
import com.example.ellegadodepintia.exploradoresDePintia.model.efecto.EfectoRiesgo
import com.example.ellegadodepintia.repositorios.RepositorioObjetos

class RiesgoFactory : EventoFactory {

    private val descripcionComun = "En un claro junto al río Duero, encuentras un viejo cofre de madera decorado con espigas de trigo y motivos solares."

    private val riesgoMap = mutableMapOf(
        1 to EventoRiesgoData(
            imagen = R.drawable.cofre_open_animation,
            opciones = listOf(
                Opcion(
                    descripcion = "Abrir el cofre",
                    atributoRequerida = Atributo.Investigacion,
                    EfectoRiesgo(
                        RepositorioObjetos.obtenerObjetoPorNombre("Espada de Hierro")!!,
                        dmgRiesgo = 2,
                        mensajeRecomensa = "Dentro del cofre encuentras una espada de hierro vaccea. Está en buen estado y tiene un brillo sobrenatural",
                        mensajeRiesgo = "Una nube de polvo tóxico se libera, causandote 2 de daño",
                        50
                    )
                ),
                Opcion(
                    descripcion = "Ignorar",
                    atributoRequerida = Atributo.Nula,
                    EfectoIgnorar(100)
                )
            )
        ),
        2 to EventoRiesgoData(
                        imagen = R.drawable.cofre_open_animation,
            opciones = listOf(
                Opcion(
                    descripcion = "Abrir el cofre",
                    atributoRequerida = Atributo.Investigacion,
                    EfectoRiesgo(
                        RepositorioObjetos.obtenerObjetoPorNombre("Ticket")!!,
                        dmgRiesgo = 4,
                        mensajeRecomensa = "Dentro del cofre encuentras un ticket. No sabes para que podría servir",
                        mensajeRiesgo = "Te encuentras una serpiente que te muerde hasta que consigues quitartela de encima, causandote 4 de daño",
                        35
                    )
                ),
                Opcion(
                    descripcion = "Ignorar",
                    atributoRequerida = Atributo.Nula,
                    EfectoIgnorar(100)
                )
            )
        )
    )

    override fun generarDescripcion(): String {
        return descripcionComun
    }

    override fun generarOpciones(descripcion: String): MutableList<Opcion> {
        val evento = riesgoMap.entries.find { it.key == descripcion.toInt() }
        return evento?.value?.opciones?.toMutableList() ?: mutableListOf()
    }

    override fun generarImagen(descripcion: String): Int {
        val evento = riesgoMap.entries.find { it.key == descripcion.toInt() }
        return evento!!.value.imagen
    }

    override fun generarEvento(): EventoRiesgo {
        val id = riesgoMap.keys.random()
        val eventoData = riesgoMap[id]!!

        val evento = EventoRiesgo(
            id = id,
            descripcion = generarDescripcion(),
            dificultad = 0,
            opciones = eventoData.opciones.toMutableList(),
            imagen = eventoData.imagen,
            resistenciaMax = (2..8).random(),
            finalizado = false,
        )

        return evento
    }
}
data class EventoRiesgoData(
    val imagen: Int,
    val opciones: List<Opcion>
)
